@@ -31,16 +31,15 @@ from ssl import CertificateError
 import time
 import email
 import codecs
-try:
-    from email.Header import decode_header
-    import email.Parser as Parser
-except ImportError:
-    from email.header import decode_header
-    import email.parser as Parser
+
+from email.header import decode_header
+import email.parser as Parser
+from email.charset import add_alias
 import poplib
 import imaplib
 import re
 import select
+import base64
 
 try:
     # do we have a recent pykerberos?
@@ -92,8 +91,8 @@ __all__ = [
     'IMAPinitMixIn',
     'IMAPRetrieverBase',
     'IMAPSSLinitMixIn',
-    'MultidropPOP3RetrieverBase',
     'MultidropIMAPRetrieverBase',
+    'MultidropPOP3RetrieverBase',
     'POP3_SSL_PORT',
     'POP3initMixIn',
     'POP3RetrieverBase',
@@ -400,10 +399,7 @@ poplib._MAXLINE = 1 << 20   # 1MB; decrease this if you're running on a VIC-20
 #######################################
 class CertMixIn(object):
     def ssl_cipher_hash(self):
-        if sys.version_info.major == 2:
-            sslobj = self.conn.sslobj
-        else:
-            sslobj = self.conn.sock
+        sslobj = self.conn.sock
         self.setup_received(sslobj)
         peercert = sslobj.getpeercert(True)
         ssl_cipher = sslobj.cipher()
@@ -880,7 +876,7 @@ class RetrieverSkeleton(ConfigurableBase):
 
     def _oldmail_filename(self, mailbox):
         assert (mailbox is None
-                or (isinstance(mailbox, (unicode, bytes)))), (
+                or (isinstance(mailbox, (str, bytes)))), (
             'bad mailbox %s (%s)' % (mailbox, type(mailbox))
         )
         filename = self.oldmail_filename
@@ -962,6 +958,7 @@ class RetrieverSkeleton(ConfigurableBase):
                 t = self.oldmail.get(msgid, self.timestamp)
                 self.log.debug(' timestamp %s' % t + os.linesep)
                 oldmailfile.write('%s\0%i%s' % (msgid, t, os.linesep))
+                self.oldmail[msgid] = t
                 wrote += 1
             oldmailfile.close()
             self.log.moreinfo('wrote %i uids for %s%s'
@@ -1025,6 +1022,9 @@ class RetrieverSkeleton(ConfigurableBase):
     def getmsg(self, msgid):
         if not self.__initialized:
             raise getmailOperationError('not initialized')
+        add_alias('cp-1252', 'cp1252')
+        add_alias('windows-31j', 'cp932')
+        add_alias('windows-874', 'cp874')
         return self._getmsgbyid(msgid)
 
     def getmsgsize(self, msgid):
@@ -1099,7 +1099,7 @@ class POP3RetrieverBase(RetrieverSkeleton):
                            % (response, octets) + os.linesep)
             for (i, line) in enumerate(msglist):
                 try:
-                    (msgnum, msgid) = tostr(line).split(None, 1)
+                    (msgnum, msgid) = line.decode().split(None, 1)
                     # Don't allow / in UIDs we store, as we look for that to
                     # detect old-style oldmail files.  Shouldn't occur in POP3
                     # anyway.
@@ -1136,7 +1136,7 @@ class POP3RetrieverBase(RetrieverSkeleton):
             self.sorted_msgnum_msgid = sorted(self.msgid_by_msgnum.items())
             (response, msglist, octets) = self.conn.list()
             for line in msglist:
-                tostrline = tostr(line)
+                tostrline = line.decode()
                 msgnum = int(tostrline.split()[0])
                 msgsize = int(tostrline.split()[1])
                 msgid = self.msgid_by_msgnum.get(msgnum, None)
@@ -1191,12 +1191,8 @@ class POP3RetrieverBase(RetrieverSkeleton):
         self.log.trace()
         msgnum = self._getmsgnumbyid(msgid)
         response, headerlist, octets = self.conn.top(msgnum, 0)
-        try:
-            parser = Parser.BytesHeaderParser()
-            return parser.parsebytes(os.linesep.encode().join(headerlist))
-        except: #py2
-            parser = Parser.HeaderParser()
-            return parser.parsestr(os.linesep.join(headerlist))
+        parser = Parser.BytesHeaderParser()
+        return parser.parsebytes(os.linesep.encode().join(headerlist))
 
     def initialize(self, options):
         self.log.trace()
@@ -1219,6 +1215,12 @@ class POP3RetrieverBase(RetrieverSkeleton):
             if self.conf['use_apop']:
                 self.conn.apop(self.conf['username'],
                                self.conf['password'])
+            elif self.conf['use_xoauth2']:
+                # octal 1 / ctrl-A used as separator
+                auth = 'user=%s\1auth=Bearer %s\1\1' % (self.conf['username'],
+                                                        self.conf['password'])
+                auth = base64.b64encode(auth.encode()).decode()
+                self.conn._shortcmd('AUTH XOAUTH2 %s' % auth)
             else:
                 self.conn.user(self.conf['username'])
                 self.conn.pass_(self.conf['password'])
@@ -1322,7 +1324,6 @@ class MultidropPOP3RetrieverBase(POP3RetrieverBase):
         msg.recipient = address_no_brackets(line.strip())
         return msg
 
-
 #######################################
 class IMAPRetrieverBase(RetrieverSkeleton):
     '''Base class for single-user IMAP mailboxes.
@@ -1346,6 +1347,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         self.msgsizes = {}
         self.oldmail = {}
         self.__delivered = {}
+        self._uidmax = 1
 
     def checkconf(self):
         RetrieverSkeleton.checkconf(self)
@@ -1390,10 +1392,19 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         self.log.trace()
         try:
             result, resplist = getattr(self.conn, cmd)(*args)
-        except imaplib.IMAP4.error as o:
+        except (imaplib.IMAP4.error, UnicodeEncodeError) as o:
             if cmd == 'login':
-                # Percolate up
-                raise
+                #raise getmailOperationError('IMAP login failed with: %s.\n'%str(o))
+                try:
+                    user, password = self.conf['username'], self.conf['password']
+                    result, resplist = self.conn.authenticate('PLAIN', lambda x: f'\0{user}\0{password}'.encode('utf-8'))
+                    cmd = "authenticate"
+                except (imaplib.IMAP4.error, UnicodeEncodeError) as oa:
+                    raise getmailOperationError(
+                        'IMAP login failed with: %s.\n\
+                        Tried IMAP authenticate, which also failed with: %s'
+                        % (str(o),str(oa))
+                    )
             else:
                 raise getmailOperationError('IMAP error (%s)' % o)
         if result != 'OK':
@@ -1403,6 +1414,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
             )
         if cmd.lower().startswith('login'):
             self.log.debug('login command response %s' % resplist + os.linesep)
+        if cmd.lower().startswith('authenticate'):
+            self.log.debug('authenticate command response %s' % resplist + os.linesep)
         else:
             self.log.debug(
                 'command %s response %s'
@@ -1451,7 +1464,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                     # Leftover part -- not name, value pair.
                     raise ValueError
                 name = parts.pop(0).lower()
-                r[tostr(name)] = tostr(parts.pop(0))
+                r[name.decode()] = parts.pop(0).decode()
         except (ValueError, IndexError, AttributeError) as o:
             raise getmailOperationError(
                 'IMAP error (failed to parse attr response line "%s": %s)'
@@ -1491,8 +1504,79 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         self.msgsizes = {}
         self.oldmail = {}
         self.__delivered = {}
+        self._uidmax = 1
         self.conn.close()
 
+    def _uidoldfile(self):
+        return self.conf['uid_cache'] and self.conf['uid_cache'].lower() == "true"
+    def _uidfile(self):
+        uidcachefile = self.conf['uid_cache']
+        if not uidcachefile:
+            return None
+        if not self._uidoldfile():
+            getmaildir = self.conf['getmaildir']
+            f = os.path.expanduser(uidcachefile)
+            if not os.path.isabs(f) and getmaildir:
+                f = os.path.join(getmaildir, f)
+                return f
+        return None
+    def _uidmaxread(self):
+        f = self._uidfile()
+        if f:
+            self._uidmax = 1
+            self._uidmax_changed = False
+            try:
+                with open(f) as C:
+                    for line in C:
+                        try:
+                            (mailbox, uidvalidity, uid) = line.split(" ")
+                            if mailbox == self.mailbox:
+                                self._uidmaxset(uidvalidity,uid)
+                        except:
+                            self._uidmaxset(None,None)
+            except FileNotFoundError:
+                pass
+        elif self._uidoldfile():
+            for msgid in list(self.oldmail):
+                (uidvalidity,uid) = msgid.split('/')
+                self._uidmaxset(uidvalidity,uid)
+    def _uidmaxwrite(self):
+        f = self._uidfile()
+        if f:
+            if not self._uidmax_changed or not self.uidvalidity:
+                return
+            uidcache = {}
+            try:
+                with open(f) as C:
+                    for line in C:
+                        mailbox, _, _ = line.split(" ")
+                        uidcache[mailbox] = line
+            except FileNotFoundError:
+                pass
+            uidvalidity = self.uidvalidity.replace(" ","_")
+            uidcache[self.mailbox] = " ".join(
+                (self.mailbox, uidvalidity, str(self._uidmax)))+"\n"
+            with open(f,"w") as C:
+                for line in uidcache.values():
+                    print(line, file=C, end="")
+    def _uidmaxset(self,uidvalidity,uid):
+        if uid is None:
+            self._uidmax = 1
+            return
+        if uidvalidity != self.uidvalidity:
+            self._uidmax = 1
+            return
+        if uidvalidity.replace(" ","_") != self.uidvalidity.replace(" ","_"):
+            self._uidmax = 1
+            return
+        try:
+            nuid = int(uid.strip())
+            if nuid > self._uidmax:
+                self._uidmax = nuid
+                self._uidmax_changed = True
+        except ValueError:
+            self._uidmax = 1
+
     def select_mailbox(self, mailbox):
         self.log.trace()
         assert mailbox in self.mailboxes, (
@@ -1507,6 +1591,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
             self.read_oldmailfile(mailbox)
 
         self.log.debug('selecting mailbox "%s"' % mailbox + os.linesep)
+        uidvalidity = None
         try:
             if (self.app_options['delete'] or self.app_options['delete_after']
                     or self.app_options['delete_bigger_than']):
@@ -1516,13 +1601,13 @@ class IMAPRetrieverBase(RetrieverSkeleton):
             if (len(mailbox) < 2 or (
                 mailbox[0],mailbox[-1]) != ('"','"')
                 ) and IMAP_ATOM_SPECIAL.search(mailbox):
-                (status, count) = self.conn.select(
+                (status, msgcount) = self.conn.select(
                     codecs.encode(
                         self.conn._quote(mailbox),
                         'imap4-utf-7'),
                     read_only)
             else:
-                (status, count) = self.conn.select(
+                (status, msgcount) = self.conn.select(
                     codecs.encode(
                         mailbox,
                         'imap4-utf-7'),
@@ -1533,8 +1618,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
 
             self.mailbox_selected = mailbox
             # use *last* EXISTS returned
-            count = int(count[-1])
-            uidvalidity = tostr(self.conn.response('UIDVALIDITY')[1][0] or b"") or None
+            msgcount = int(msgcount[-1])
+            uidvalidity = self.conn.response('UIDVALIDITY')[1][0]
         except imaplib.IMAP4.error as o:
             raise getmailOperationError('IMAP error (%s)' % o)
         except (IndexError, ValueError) as o:
@@ -1543,35 +1628,56 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                 % o
             )
         self.log.debug('select(%s) returned message count of %d'
-                       % (mailbox, count) + os.linesep)
+                       % (mailbox, msgcount) + os.linesep)
         self.mailbox = mailbox
-        self.uidvalidity = uidvalidity
-
+        self.uidvalidity = uidvalidity and uidvalidity.decode() or None
         imap_search = self.conf['imap_search']
         if imap_search:
             result, data = self.conn.search(None, imap_search)
             data = [d for d in data if d is not None]
             if result == 'OK' and data:
                 msgs = b','.join(data[0].split())
-                if sys.version_info.major > 2:
-                    msgs = msgs.decode()
+                msgs = msgs.decode()
                 self._getmsglist(msgs)
         else:
-            self._getmsglist(count)
+            self._uidmaxread()
+            self._getmsglist(msgcount, start=self._uidmax)
+            self._uidmaxwrite()
 
-        return count
+        return msgcount
 
-    def _getmsglist(self, msgcount):
+    def _remove_from_oldmail(self):
+        # Remove messages from the oldmail state file that are no longer in mailbox,
+        # but only if the timestamp for them are old (30 days for now).
+        # This is because IMAP users can have one oldmail state file but multiple
+        # IMAP folders in different configuration rc files.
+        for msgid in list(self.oldmail):
+            timestamp = self.oldmail[msgid]
+            age = self.timestamp - timestamp
+            if msgid not in self.msgsizes and age > VANISHED_AGE:
+                self.log.debug('removing vanished old message id %s' % msgid
+                                + os.linesep)
+                del self.oldmail[msgid]
+
+    def _getmsglist(self, msgs, start=1):
         self.log.trace()
         try:
-            if msgcount:
+            if msgs:
                 # Get UIDs and sizes for all messages in mailbox
-                response = self._parse_imapcmdresponse(
-                    'FETCH',
-                    isinstance(msgcount,str) and msgcount or '1:%d'%msgcount,
-                    ('(UID)' if self.app_options['skip_imap_fetch_size'] else
-                    '(UID RFC822.SIZE)')
-                )
+                remove_from_oldmail = False
+                if start > 1:
+                    fetchcmd=['UID', 'FETCH']
+                    fetchcmd.append("%d:*"%(start+1))
+                else:
+                    fetchcmd=['FETCH']
+                    if isinstance(msgs,str):
+                        fetchcmd.append(msgs)
+                    else:
+                        fetchcmd.append('1:%d'%msgs)
+                        remove_from_oldmail = True
+                fetchcmd.append(('(UID)' if self.app_options['skip_imap_fetch_size'] else
+                                 '(UID RFC822.SIZE)'))
+                response = self._parse_imapcmdresponse(*fetchcmd)
                 for line in response:
                     if not line:
                         # One user had a server that returned a null response
@@ -1614,6 +1720,9 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                     # detect old-style oldmail files.  Can occur with IMAP, at
                     # least with some servers.
                     uid = r['uid'].replace('/', '-')
+
+                    self._uidmaxset(self.uidvalidity, uid)
+
                     msgid = '%s/%s' % (self.uidvalidity, uid)
                     self._mboxuids[msgid] = r['uid']
                     self._mboxuidorder.append(msgid)
@@ -1621,19 +1730,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                     self.msgsizes[msgid] = (0 if
                         self.app_options['skip_imap_fetch_size']
                                         else int(r['rfc822.size']))
-
-            # Remove messages from state file that are no longer in mailbox,
-            # but only if the timestamp for them are old (30 days for now).
-            # This is because IMAP users can have one state file but multiple
-            # IMAP folders in different configuration rc files.
-            for msgid in list(self.oldmail):
-                timestamp = self.oldmail[msgid]
-                age = self.timestamp - timestamp
-                if msgid not in self.msgsizes and age > VANISHED_AGE:
-                    self.log.debug('removing vanished old message id %s' % msgid
-                                   + os.linesep)
-                    del self.oldmail[msgid]
-
+                if remove_from_oldmail:
+                    self._remove_from_oldmail()
         except imaplib.IMAP4.error as o:
             raise getmailOperationError('IMAP error (%s)' % o)
         self.gotmsglist = True
@@ -1827,24 +1925,6 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                     raise getmailCredentialError(o)
 
             self.log.trace('logged in' + os.linesep)
-            """
-            self.log.trace('logged in, getting message list' + os.linesep)
-            self._getmsglist()
-            self.log.debug('msgids: %s'
-                           % list(sorted(self.msgnum_by_msgid.keys())) + os.linesep)
-            self.log.debug('msgsizes: %s' % self.msgsizes + os.linesep)
-            # Remove messages from state file that are no longer in mailbox,
-            # but only if the timestamp for them are old (30 days for now).
-            # This is because IMAP users can have one state file but multiple
-            # IMAP folders in different configuration rc files.
-            for msgid in self.oldmail:
-                timestamp = self.oldmail[msgid]
-                age = self.timestamp - timestamp
-                if msgid not in self.msgsizes and age > VANISHED_AGE:
-                    self.log.debug('removing vanished old message id %s' % msgid
-                                   + os.linesep)
-                    del self.oldmail[msgid]
-            """
             # Some IMAP servers change the available capabilities after
             # authentication, i.e. they present a limited set before login.
             # The Python stlib IMAP4 class doesn't take this into account
@@ -1855,7 +1935,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                 # No response, don't update the stored capabilities
                 self.log.warning('no post-login CAPABILITY response from server\n')
             else:
-                self.conn.capabilities = tuple(tostr(dat[-1]).upper().split())
+                self.conn.capabilities = tuple(dat[-1].decode().upper().split())
 
             if 'IDLE' in self.conn.capabilities:
                 self.supports_idle = True
@@ -1920,7 +2000,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
 
         try:
             aborted = None
-            (readable, unused, unused) = select.select([self.conn.sock], [], [], timeout)
+            (readable, unused, _) = select.select([self.conn.sock], [], [], timeout)
         except KeyboardInterrupt as o:
             # Delay raising this until we've stopped IDLE mode
             aborted = o
@@ -1937,6 +2017,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
             self.conn.untagged_responses = {}
             self.conn.send(b'DONE\r\n')
             self.conn._command_complete('IDLE', tag)
+        except ssl.SSLEOFError as o:
+            return False
         except imaplib.IMAP4.error as o:
             return False
         except BrokenPipeError as o:
@@ -2027,4 +2109,3 @@ class MultidropIMAPRetrieverBase(IMAPRetrieverBase):
             )
         msg.recipient = address_no_brackets(line.strip())
         return msg
-
